from hwsis.utils.logger_utils import logger


class HotWordRequest:
    def __init__(self, name, word_list):
        self._name = name
        self._word_list = word_list
        self._language = 'chinese_mandarin'
        self._description = ''

    def set_description(self, description):
        self._description = description

    def set_language(self, language):
        self._language = language

    def construct_params(self):
        params_dict = {
            'name': self._name,
            'language': self._language,
            'description': self._description,
            'contents': self._word_list
        }
        return params_dict



# -*- coding: utf-8 -*-

import requests
import json
from hwsis.utils.logger_utils import logger
from hwsis.exception.exceptions import ClientException, ServerException
requests.packages.urllib3.disable_warnings()
NUM_MAX_RETRY = 5


def http_connect(url, header, data, http_method='POST', time_out=5, proxy=None):
    """
        post请求，带有header信息（用于认证）
    :param url: -
    :param header: 头部
    :param data: post数据
    :param time_out: 超时
    :param proxy: 代理
    :param http_method: http方法，目前支持put、delete、post、get
    :return: http请求的response
    """
    if isinstance(data, dict):
        data = json.dumps(data)
    if proxy is not None:
        proxy = _generate_request_proxy(proxy)
    else:
        proxy = {
            'http': None,
            'https': None
        }
    # 加入重试机制
    count = 0
    resp = None
    while count < NUM_MAX_RETRY:
        try:
            if http_method == 'POST':
                resp = requests.post(url, headers=header, data=data, timeout=time_out, verify=False, proxies=proxy)
            elif http_method == 'GET':
                resp = requests.get(url, headers=header, params=data, timeout=time_out, verify=False, proxies=proxy)
            elif http_method == 'PUT':
                resp = requests.put(url, headers=header, data=data, timeout=time_out, verify=False, proxies=proxy)
            elif http_method == 'DELETE':
                resp = requests.delete(url, headers=header, params=data, timeout=time_out, verify=False, proxies=proxy)
            else:
                logger.error('%s is invalid' % http_method)
                raise ClientException('%s is invalid' % http_method)
            break
        except requests.exceptions.RequestException as e:
            logger.error('Error occurs in %s, the client will retry 5 times. Error message is %s' % (http_method, e))
            count += 1
    if resp is None:
        logger.error('%s Response is empety, url is %s' % (http_method, url))
        raise ClientException('%s Response is empety, url is %s' % (http_method, url))
    return resp


def parse_resp(resp):
    """
        requests响应转化为json格式
    :param resp: requests请求返回的响应
    :return: json
    """
    if resp is None or resp.text is None or resp.text == '':
        return None
    text = resp.text
    try:
        result = json.loads(text)
    except Exception as e:
        error_msg = 'Parsing json failed, the text is %s' % text
        logger.error(error_msg)
        raise ClientException(error_msg)
    if 'error_code' in result and 'error_msg' in result:
        error_msg = json.dumps(result)
        logger.error(error_msg)
        raise ServerException(result['error_code'], result['error_msg'])
    return result


def generate_scheme_host_uri(url):
    if url.find('//') == -1 or url.find('com') == -1:
        error_msg = '%s is invalid' % url
        logger.error(error_msg)
        raise ClientException(error_msg)
    split1s = url.split('//')
    split2s = split1s[1].split('com')
    scheme = split1s[0] + '//'
    host = split2s[0] + 'com'
    uri = split2s[1]
    return scheme, host, uri


def _generate_request_proxy(proxy):
    if proxy is None:
        return proxy
    if not isinstance(proxy, list) or (not len(proxy) == 2 and not len(proxy) == 4):
        logger.error('Proxy must be list, the format is [host, port] or [host, port, username, password]')
        raise ClientException('Proxy must be list, the format is [host, port] or [host, port, username, password]')
    proxy_str = str(proxy[0]) + ':' + str(proxy[1])
    if len(proxy) == 2:
        proxy = {
            'http': 'http://' + proxy_str,
            'https': 'https://' + proxy_str
        }
    else:
        proxy = {
            'http': 'http://' + str(proxy[2]) + ':' + str(proxy[3]) + '@' + proxy_str,
            'https': 'https://' + str(proxy[2]) + ':' + str(proxy[3]) + '@' + proxy_str
        }
    return proxy



from hwsis.utils.logger_utils import logger
from hwsis.exception.exceptions import ClientException


class PaAudioRequest:
    """ 语音评测请求 """
    def __init__(self):
        """
            语音请求初始化
        """
        self._audio_data = None
        self._audio_url = None
        self._ref_text = None
        self._audio_format = 'auto'
        self._language = 'en_gb'
        self._mode = 'word'

    def set_audio_data(self, audio_data):
        self._audio_data = audio_data

    def set_audio_url(self, audio_url):
        self._audio_url = audio_url

    def set_audio_format(self, audio_format):
        self._audio_format = audio_format

    def set_ref_text(self, ref_text):
        self._ref_text = ref_text

    def set_language(self, language):
        self._language = language

    def set_mode(self, mode):
        self._mode = mode

    def construct_params(self):
        if self._audio_data is None and self._audio_url is None:
            logger.error('In PaAudioRequest, audio_data and audio_url can\'t be both empty')
            raise ClientException('In PaAudioRequest, audio_data and audio_url can\'t be both empty')
        if self._ref_text is None:
            logger.error('In PaAudioRequest, ref_text can\'t be empty')
            raise ClientException('In PaAudioRequest, ref_text can\'t be empty')
        if self._audio_data is not None and self._audio_url is not None:
            logger.warn('When audio_data and audio_url are all filled, only audio_data takes effect')
        params_dict = {
            'ref_text': self._ref_text,
            'config': {
                'audio_format': self._audio_format,
                'language': self._language,
                'mode': self._mode
            }
        }
        if self._audio_data is not None:
            params_dict['audio_data'] = self._audio_data
        else:
            params_dict['audio_url'] = self._audio_url
        return params_dict


class PaVideoRequest:
    def __init__(self):
        """
            多模态评测请求

        """
        self._video_data = None
        self._video_url = None
        self._video_format = 'auto'
        self._ref_text = None
        self._language = 'en_gb'
        self._mode = 'word'

    def set_video_data(self, video_data):
        self._video_data = video_data

    def set_video_url(self, video_url):
        self._video_url = video_url

    def set_video_format(self, video_format):
        self._video_format = video_format

    def set_ref_text(self, ref_text):
        self._ref_text = ref_text

    def set_language(self, language):
        self._language = language

    def set_mode(self, mode):
        self._mode = mode

    def construct_parameter(self):
        if self._video_data is None and self._video_url is None:
            logger.error('In PaVideoRequest, video_data and video_url can\'t be both empty')
            raise ClientException('In PaVideoRequest, video_data and video_url can\'t be both empty')
        if self._ref_text is None:
            logger.error('In PaVideoRequest, ref_text can\'t be empty')
            raise ClientException('In PaVideoRequest, ref_text can\'t be empty')
        if self._video_data is not None and self._video_url is not None:
            logger.warn('When video_data and video_url are all filled, only video_data takes effect')
        params_dict = {
            'ref_text': self._ref_text,
            'config': {
                'video_format': self._video_format,
                'language': self._language,
                'mode': self._mode
            }
        }
        if self._video_data is not None:
            params_dict['video_data'] = self._video_data
        else:
            params_dict['video_url'] = self._video_url
        return params_dict



# -*- coding: utf-8 -*-

import json
from hwsis.auth import aksk_service
from hwsis.bean.tts_request import TtsCustomRequest
from hwsis.utils import io_utils
from hwsis.exception.exceptions import ClientException
from hwsis.bean.sis_config import SisConfig
from hwsis.utils.logger_utils import logger


class TtsCustomizationClient:
    """ 语音合成client """
    def __init__(self, ak, sk, region, project_id, service_endpoint=None, sis_config=None):
        self._ak = ak
        self._sk = sk
        self._region = region
        self._project_id = project_id
        if service_endpoint is None:
            self._service_endpoint = 'https://sis-ext.' + region + '.myhuaweicloud.com'
        else:
            self._service_endpoint = service_endpoint
        if sis_config is None:
            self._sis_config = SisConfig()
        else:
            self._sis_config = sis_config

    def get_ttsc_response(self, request):
        """
            语音合成接口
        :param request: 语音合成请求，TtsCustomRequest
        :return: 请求结果，json格式
        """
        if not isinstance(request, TtsCustomRequest):
            logger.error('the parameter in \'get_ttsc_response(request)\' should be TtsCustomRequest class')
            raise ClientException('the parameter in \'get_ttsc_response(request)\' should be TtsCustomRequest class')
        url = self._service_endpoint + '/v1/' + self._project_id + '/tts'
        params = request.construct_params()
        headers = {'Content-Type': 'application/json'}
        result = aksk_service.aksk_connect(self._ak, self._sk, url, headers, params, 'POST', self._sis_config)
        if 'result' not in result:
            error_msg = 'The result of tts customization is invalid. Result is %s ' % json.dumps(result)
            logger.error(error_msg)
            raise ClientException(error_msg)
        if request.get_saved():
            base_str = result['result']['data']
            io_utils.save_audio_from_base64str(base_str, request.get_saved_path())
            result['is_saved'] = True
            result['saved_path'] = request.get_saved_path()
        return result



# -*- coding: utf-8 -*-

import base64
import os
from hwsis.utils.logger_utils import logger
from hwsis.exception.exceptions import ClientException


def encode_file(file_path):
    if not os.path.exists(file_path):
        logger.error('The Path %s doesn\'t exist' % file_path)
        raise ClientException('The Path %s doesn\'t exist' % file_path)
    with open(file_path, 'rb') as f:
        data = f.read()
        base64_data = str(base64.b64encode(data), 'utf-8')
        return base64_data


def save_audio_from_base64str(base64_str, save_path):
    parent_path = os.path.dirname(save_path)
    if parent_path != '' and not os.path.exists(parent_path):
        os.makedirs(parent_path)
    with open(save_path, 'wb') as f:
        base64_data = base64.b64decode(base64_str)
        f.write(base64_data)



# -*- coding: utf-8 -*-

from hwsis.auth import aksk_service
from hwsis.bean.asr_request import AsrCustomLongRequest
from hwsis.bean.asr_request import AsrCustomShortRequest
from hwsis.utils.logger_utils import logger
from hwsis.exception.exceptions import ClientException
from hwsis.bean.sis_config import SisConfig


class AsrCustomizationClient:
    """ 语音识别client """
    def __init__(self, ak, sk, region, project_id, service_endpoint=None, sis_config=None):
        """
            语音识别client初始化
        :param ak:                  ak
        :param sk:                  sk
        :param region:              区域，如cn-north-4
        :param project_id:          项目id，可参考https://support.huaweicloud.com/api-sis/sis_03_0008.html
        :param service_endpoint:    终端节点，可不填使用默认即可
        :param sis_config:          配置信息，包括超时、代理等，可不填使用默认即可。
        """
        self._ak = ak
        self._sk = sk
        self._region = region
        self._project_id = project_id
        if service_endpoint is None:
            self._service_endpoint = 'https://sis-ext.' + region + '.myhuaweicloud.com'
        else:
            self._service_endpoint = service_endpoint
        if sis_config is None:
            self._sis_config = SisConfig()
        else:
            self._sis_config = sis_config

    def get_short_response(self, request):
        """
            一句话识别接口
        :param request: 一句话识别请求AsrCustomShortRequest
        :return: 一句话识别响应结果，返回为json格式
        """
        if not isinstance(request, AsrCustomShortRequest):
            error_msg = 'the parameter in \'get_short_response(request)\' should be AsrCustomShortRequest class'
            logger.error(error_msg)
            raise ClientException(error_msg)
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/short-audio'
        params = request.construct_params()
        headers = {'Content-Type': 'application/json'}
        result = aksk_service.aksk_connect(self._ak, self._sk, url, headers, params, 'POST', self._sis_config)
        return result

    def submit_job(self, request):
        """
            录音文件识别，提交任务接口
        :param request: 录音文件识别请求
        :return: job_id
        """
        if not isinstance(request, AsrCustomLongRequest):
            error_msg = 'the parameter in \'submit_job(request)\' should be AsrCustomLongRequest class'
            logger.error(error_msg)
            raise ClientException(error_msg)
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/transcriber/jobs'
        headers = {'Content-Type': 'application/json'}
        params = request.construct_parameter()
        result = aksk_service.aksk_connect(self._ak, self._sk, url, headers, params, 'POST', self._sis_config)
        if 'job_id' not in result:
            error_msg = 'The result of long audio transcription doesn\'t contain key job_id, result is ' % result
            logger.error(error_msg)
            raise ClientException(error_msg)
        return result['job_id']

    def get_long_response(self, job_id):
        """
            录音文件识别状态查询接口
        :param job_id: job_id
        :return: 返回的结果，json格式
        """
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/transcriber/jobs/' + job_id
        result = aksk_service.aksk_connect(self._ak, self._sk, url, None, None, 'GET', self._sis_config)
        return result



# -*- coding: utf-8 -*-

from hwsis.bean.hot_word_request import HotWordRequest
from hwsis.bean.sis_config import SisConfig
from hwsis.utils.logger_utils import logger
from hwsis.auth import aksk_service
from hwsis.exception.exceptions import ClientException, ServerException
import json


class HotWordClient:
    """ 热词client，可用于创建热词表、更新热词表、查询热词表列表、查询热词表、删除热词表 """
    def __init__(self, ak, sk, region, project_id, service_endpoint=None, sis_config=None):
        """
            口语评测client初始化
        :param ak:                  ak
        :param sk:                  sk
        :param region:              区域，如cn-north-4
        :param project_id:          项目id，可参考https://support.huaweicloud.com/api-sis/sis_03_0008.html
        :param service_endpoint:    终端节点，可不填使用默认即可
        :param sis_config:          配置信息，包括超时、代理等，可不填使用默认即可。
        """
        self._ak = ak
        self._sk = sk
        self._region = region
        self._project_id = project_id
        if service_endpoint is None:
            self._service_endpoint = 'https://sis-ext.' + region + '.myhuaweicloud.com'
        else:
            self._service_endpoint = service_endpoint
        if sis_config is None:
            self._sis_config = SisConfig()
        else:
            self._sis_config = sis_config

    def create(self, request):
        """
            创建热词表
        :param request: 热词表请求
        :return: 热词表id
        """
        if not isinstance(request, HotWordRequest):
            logger.error('the parameter in \'create_hot_word(request)\' should be HotWordRequest class')
            raise ClientException('the parameter in \'create_hot_word(request)\' should be HotWordRequest class')
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/vocabularies'
        headers = {'Content-Type': 'application/json'}
        params = request.construct_params()
        result = aksk_service.aksk_connect(self._ak, self._sk, url, headers, params, 'POST', self._sis_config)
        return result

    def update(self, request, vocabulary_id):
        """
            更新热词表
        :param request: 热词表请求
        :param vocabulary_id: 热词表id，更新时一定要保证该热词表存在
        :return: 热词表id
        """
        if not isinstance(request, HotWordRequest):
            logger.error('the parameter in \'update_hot_word(request)\' should be HotWordRequest class')
            raise ClientException('the parameter in \'update_hot_word(request)\' should be HotWordRequest class')
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/vocabularies/' + vocabulary_id
        headers = {'Content-Type': 'application/json'}
        params = request.construct_params()
        result = aksk_service.aksk_connect(self._ak, self._sk, url, headers, params, 'PUT', self._sis_config)
        return result

    def query_list(self):
        """
            查询热词表列表信息
        :return: 热词表列表信息
        """
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/vocabularies'
        result = aksk_service.aksk_connect(self._ak, self._sk, url, None, None, 'GET', self._sis_config)
        return result

    def query_by_vocabulary_id(self, vocabulary_id):
        """
            根据vocabulary_id查询热词表信息
        :param vocabulary_id: 热词表id，使用前一定要保证其已存在。
        :return:  热词表信息
        """
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/vocabularies/' + vocabulary_id
        result = aksk_service.aksk_connect(self._ak, self._sk, url, None, None, 'GET', self._sis_config)
        return result

    def delete(self, vocabulary_id):
        """
            根据vocabulary_id删除指定的热词表
        :param vocabulary_id: 热词表id，使用前一定要保证其已存在。
        :return: 正常删除返回结果为空，出现错误则返回error_code和error_msg
        """
        url = self._service_endpoint + '/v1/' + self._project_id + '/asr/vocabularies/' + vocabulary_id
        result = aksk_service.aksk_connect(self._ak, self._sk, url, None, None, 'DELETE', self._sis_config)
        if result is None:
            return result
        result_text = result.text
        return json.loads(result_text)



# -*- coding: utf-8 -*-


class AsrCustomShortRequest:
    """ 一句话识别请求，除了初始化必选参数外，其他参数均可不配置使用默认 """
    def __init__(self, audio_format, model_property, data):
        """
            一句话识别请求初始化
        :param audio_format:    音频格式，详见api文档
        :param model_property:  language_sampleRate_domain, 如chinese_8k_common, 详见api文档
        :param data:            音频转化后的base64字符串
        """
        self._audio_format = audio_format
        self._property = model_property
        self._data = data
        self._add_punc = 'no'
        self._digit_norm = 'yes'
        self._vocabulary_id = None
        self._need_word_info = 'no'

    def set_add_punc(self, add_punc):
        self._add_punc = add_punc

    def set_digit_norm(self, digit_norm):
        self._digit_norm = digit_norm

    def set_vocabulary_id(self, vocabulary_id):
        self._vocabulary_id = vocabulary_id

    def set_need_word_info(self, need_word_info):
        self._need_word_info = need_word_info

    def construct_params(self):
        params = dict()
        params['data'] = self._data
        config = dict()
        config['audio_format'] = self._audio_format
        config['property'] = self._property
        config['add_punc'] = self._add_punc
        config['digit_norm'] = self._digit_norm
        config['need_word_info'] = self._need_word_info
        if self._vocabulary_id is not None:
            config['vocabulary_id'] = self._vocabulary_id
        params['config'] = config
        return params


class AsrCustomLongRequest:
    """ 录音文件识别请求，除了初始化必选参数外，其他参数均可不配置使用默认 """
    def __init__(self, audio_format, model_property, data_url):
        """
            录音文件识别初始化
        :param audio_format:   音频格式，详见api文档
        :param model_property: 属性字符串，language_sampleRate_domain, 详见api文档
        :param data_url:       音频的obs链接
        """
        self._audio_format = audio_format
        self._property = model_property
        self._data_url = data_url
        self._add_punc = 'no'
        self._digit_norm = 'yes'
        self._callback_url = None
        self._need_analysis_info = False
        self._diarization = True
        self._channel = 'MONO'
        self._emotion = True
        self._speed = True
        self._vocabulary_id = None
        self._need_word_info = 'no'

    def set_callback_url(self, callback_url):
        self._callback_url = callback_url

    def set_add_punc(self, add_punc):
        self._add_punc = add_punc

    def set_digit_norm(self, digit_norm):
        self._digit_norm = digit_norm

    def set_need_analysis_info(self, need_analysis_info):
        self._need_analysis_info = need_analysis_info

    def set_diarization(self, diarization):
        self._diarization = diarization

    def set_channel(self, channel):
        self._channel = channel

    def set_emotion(self, emotion):
        self._emotion = emotion

    def set_speed(self, speed):
        self._speed = speed

    def set_vocabulary_id(self, vocabulary_id):
        self._vocabulary_id = vocabulary_id

    def set_need_word_info(self, need_word_info):
        self._need_word_info = need_word_info

    def construct_parameter(self):
        params = dict()
        params['data_url'] = self._data_url
        config = dict()
        config['audio_format'] = self._audio_format
        config['property'] = self._property
        config['add_punc'] = self._add_punc
        config['digit_norm'] = self._digit_norm
        config['need_word_info'] = self._need_word_info
        if self._callback_url is not None and not self._callback_url == '':
            config['callback_url'] = self._callback_url
        if self._need_analysis_info:
            need_analysis_info = dict()
            need_analysis_info['diarization'] = self._diarization
            need_analysis_info['channel'] = self._channel
            need_analysis_info['emotion'] = self._emotion
            need_analysis_info['speed'] = self._speed
            config['need_analysis_info'] = need_analysis_info
        if self._vocabulary_id is not None:
            config['vocabulary_id'] = self._vocabulary_id
        params['config'] = config
        return params



# -*- coding: utf-8 -*-

from hwsis.utils import http_utils
from hwsis.bean.sis_config import SisConfig
from hwsis.exception.exceptions import ClientException
from hwsis.utils.logger_utils import logger


def get_token(user_name, password, domain_name, region, url=None, config=SisConfig()):
    """
        获取token
    :param user_name:   用户名
    :param password:    密码
    :param domain_name: 账户名，一般等同用户名
    :param region:      区域，如cn-north-4
    :param url:         请求token的url，可使用默认值
    :param config       配置信息
    :return:            请求的token
    """
    if url is None:
        url = 'https://iam.' + region + '.myhuaweicloud.com/v3/auth/tokens'
    if not isinstance(config, SisConfig):
        error_msg = 'the param \'config\' in token_service must be SisConfig class'
        logger.error(error_msg)
        raise ClientException(error_msg)
    time_out = (config.get_connect_timeout(), config.get_read_timeout())
    proxy = config.get_proxy()
    auth_data = {
        "auth": {
            "identity": {
                "password": {
                    "user": {
                        "name": user_name,
                        "password": password,
                        "domain": {
                            "name": domain_name
                        }
                    }
                },
                "methods": [
                    "password"
                ]
            },
            "scope": {
                "project": {
                    "name": region
                }
            }
        }
    }

    headers = {'Content-Type': 'application/json'}
    req = http_utils.http_connect(url, headers, auth_data, 'POST', time_out, proxy)
    if 'X-Subject-Token' not in req.headers:
        logger.error('Error occurs in getting token, %s' % req.text)
        raise ClientException('Error occurs in getting token, %s' % req.text)
    token = req.headers['X-Subject-Token']
    return token



# -*- coding: utf-8 -*-

import websocket
from hwsis.utils.logger_utils import logger
from hwsis.auth import token_service
from hwsis.exception.exceptions import ClientException, ServerException
from hwsis.bean.callback import RttsCallBack
from hwsis.bean.sis_config import SisConfig
import ssl
from hwsis.bean.rtts_request import RttsRequest
from hwsis.auth import aksk_service
import json
import time
import threading

user_dict = dict()
time_dict = dict()

connect_sleep_time = 0.01


class RttsClient:
    """ 实时语音合成client """

    def __init__(self, user_name=None, password=None, domain_name=None, region=None, project_id=None, callback=None,
                 config=SisConfig(), service_endpoint=None, token_url=None, retry_sleep_time=1,
                 ak=None, sk=None, use_aksk=False):
        """
            实时语音合成client初始化。 推荐使用ak、sk认证方式。
        :param user_name:           用户名，当use_aksk为false生效.
        :param password:            密码，当use_aksk为false生效
        :param domain_name:         账户名，一般等同用户名，当use_aksk为false生效
        :param region:              区域，如cn-north-4
        :param project_id:          项目ID，可参考https://support.huaweicloud.com/api-sis/sis_03_0008.html
        :param callback:            回调类RttsCallBack，用于监听websocket连接、响应、断开、错误等
        :param service_endpoint:    终端节点，一般使用默认即可
        :param token_url:           请求token的url，一般使用默认即可
        :param retry_sleep_time:    当websocket连接失败重试的间隔时间，默认为1s
        :param ak                   ak, 当 use_aksk为true生效。
        :param sk                   sk, 当 use_aksk为true生效。
        :param use_aksk             是否使用ak sk认证，为true表示选择ak sk认证。为false表示使用token认证。
        """
        if service_endpoint is None:
            self._service_endpoint = 'wss://sis-ext.' + region + '.myhuaweicloud.com'
        else:
            self._service_endpoint = service_endpoint
        if not isinstance(callback, RttsCallBack):
            logger.error('The parameter callback must be RttsCallBack class')
            raise ClientException('The parameter callback must be RttsCallBack class')
        if not isinstance(config, SisConfig):
            logger.error('The parameter config must by SisConfig class')
            raise ClientException('The parameter config must by SisConfig class')

        self._region = region
        self._project_id = project_id
        self._config = config
        self._callback = callback
        self._status = 'pre_start'
        self._request = None
        self._retry_sleep_time = retry_sleep_time

        self._use_aksk = use_aksk
        if use_aksk:
            if ak is None or sk is None:
                logger.error('you choose aksk authentication, ak or sk is empty, please fill in it')
                raise ClientException('ak or sk is empty, please fill in it')
            self._ak = ak
            self._sk = sk
        else:
            if user_name is None or password is None or domain_name is None:
                logger.error('you choose token authentication, username or password or domain name is empty, '
                             'please fill in it')
                raise ClientException('username or password or domain name is empty, please fill in it')
            self._token = self._get_token(user_name, password, domain_name, token_url)


        # 此变量正常情况下永远为False。除非依赖库websocket-cliet变更，不再自动添加header
        # 1. websocket的库会在请求头部自动添加host信息。而且websocket库的header格式为tuple，即 ['key1:val1', 'key2:val2']
        # 2. apig加密的header也包含host信息。
        # 3. 当将apig加密后的header放在websocket请求中，会形成两个host信息，且无法覆盖。导致无法通过apig鉴权，报错401。
        # 4. 将其设置为false，可以屏蔽apig加密header里的host信息，最终形成header只有一个host，可通过鉴权。
        self._show_host_header = False

    def _connect(self, url):
        def _check_result(result):
            result_str = json.dumps(result)
            if 'error_code' in result and 'error_msg' in result:
                logger.error(result_str)
                # 睡眠2s保证关闭前，发送端有充足时间收到关闭请求。
                self._status = 'close'
                self._callback.on_error(result_str)
                time.sleep(2)
                self._ws.close()
                raise ServerException(result['error_code'], result['error_msg'])
            if 'resp_type' not in result:
                self._status == 'error'
                error_msg = 'result doesn\'t contain key resp, result is %s' % result_str
                logger.error(error_msg)
                self._callback.on_error(error_msg)
                raise ClientException(error_msg)

        def _on_open(ws):
            logger.info('websocket connect')
            self._status = 'connect'
            self._callback.on_open()

        def _on_message(ws, message):
            if isinstance(message, bytes):
                self._callback.on_response(data=message)
            elif isinstance(message, str):
                result = json.loads(message)
                _check_result(result)
                result_type = result['resp_type']
                trace_id = result['trace_id']
                if result_type == 'START':
                    self._status = 'start'
                    self._callback.on_start('trace id is %s' % trace_id)
                elif result_type == 'END':
                    self._status = 'end'
                    self._callback.on_end('trace id is %s' % trace_id)
                else:
                    logger.error('%s don\'t belong to any type' % result_type)
            else:
                logger.error('don\'t support message type {}', str(message))

        def _on_close(ws):
            logger.info('websocket close')
            self._status = 'close'
            self._callback.on_close()

        def _on_error(ws, error):
            logger.error(error)
            self._status = 'error'
            self._callback.on_error(error)

        sslopt = {"cert_reqs": ssl.CERT_NONE}
        headers = self._get_headers(url)
        retry_count = 5
        for i in range(retry_count):
            self._status = 'pre_start'
            self._ws = websocket.WebSocketApp(url, headers, on_open=_on_open, on_close=_on_close,
                                              on_message=_on_message, on_error=_on_error)

            self._thread = threading.Thread(target=self._ws.run_forever,
                                            args=(None, sslopt, self._config.get_connect_lost_timeout() + 10,
                                                  self._config.get_connect_lost_timeout()))
            self._thread.daemon = True
            self._thread.start()

            connect_count = int(self._config.get_connect_timeout() / connect_sleep_time)
            for j in range(connect_count):
                if self._status != 'pre_start':
                    break
                else:
                    time.sleep(connect_sleep_time)
            if self._status == 'start' or self._status == 'connect':
                break
            else:
                logger.error('connect meets error, it will retry %d times, now it is %d' % (retry_count, i + 1))
            time.sleep(self._retry_sleep_time)
        if self._status == 'pre_start' or self._status == 'close' or self._status == 'error' or self._status == 'end':
            logger.error('websocket connect failed， url is %s' % url)
            raise ClientException('websocket connect failed， url is %s' % url)

    def send_request(self, request):
        """ 发送合成请求 """
        if self._status != 'pre_start':
            logger.error("status %s is invalid. One client can only send one request, you should create a"
                         " new client to send request" % self._status)
            return
        self._check_request(request)
        self._request = request
        self._connect_to_websocket()
        message = json.dumps(self._request.construct_params())
        self._ws.send(message, opcode=websocket.ABNF.OPCODE_TEXT)

    def close(self):
        """ 服务超过20s没收到请求会自动断开 """
        count = 0  # 20s
        while self._status != 'end' and count < 200:
            time.sleep(0.1)
            count += 1
            if self._status == 'error' or self._status == 'close':
                break
        if self._thread and self._thread.is_alive():
            self._ws.keep_running = False
            self._thread.join()
        self._ws.close()

    def _connect_to_websocket(self):
        url = self._service_endpoint + '/v1/' + self._project_id + '/rtts'
        self._connect(url)

    def _set_host_header(self, show_host_header):
        """
            正常可访问情况下，永远不要调用这个接口。默认屏蔽，除非依赖库websocket-client变更，不再自动添加header中的host信息。
        :return:
        """
        self._show_host_header = show_host_header


    @staticmethod
    def _check_request(request):
        if not isinstance(request, RttsRequest):
            error_msg = 'The parameter of request in RttsClient should be RttsRequest class'
            logger.error(error_msg)
            raise ClientException(error_msg)

    def _get_headers(self, url):
        if self._use_aksk:
            if not url.startswith('ws'):
                logger.error('%s is invalid' % url)
                raise ClientException('%s is invalid' % url)
            new_url = 'http' + url[2:]
            headers = aksk_service.get_signed_headers(self._ak, self._sk, new_url, None, None, 'GET')
            if 'host' in headers and not self._show_host_header:
                del headers['host']
        else:
            headers = {'X-Auth-Token': self._token}
        return headers

    def _get_token(self, user_name, password, domain_name, token_url):
        if self._use_aksk:
            return None
        if token_url is None:
            token_url = 'https://iam.' + self._region + '.myhuaweicloud.com/v3/auth/tokens'
        # token 缓存必须在client进行，才可以在多线程中生效。
        now_time = time.time()
        token = None
        if user_name in user_dict and user_name in time_dict:
            temp_token = user_dict[user_name]
            save_time = time_dict[user_name]
            if now_time - save_time < 5 * 3600:
                logger.info('use token cache')
                token = temp_token
        if token is None:
            token = token_service.get_token(user_name, password, domain_name, self._region,
                                            url=token_url, config=self._config)
            user_dict[user_name] = token
            time_dict[user_name] = now_time
        return token



# -*- coding: utf-8 -*-


class RasrRequest:
    """ 实时语音识别请求，除了初始化必选参数外，其他参数均可不配置使用默认 """
    def __init__(self, audio_format, model_property):
        """
            实时语音识别请求初始化
        :param audio_format:   音频格式，详见api文档
        :param model_property: 属性字符串，language_sampleRate_domain, 如chinese_8k_common，详见api文档
        """
        self._audio_format = audio_format
        self._property = model_property
        self._add_punc = 'no'
        self._vad_head = 10000
        self._vad_tail = 500
        self._max_seconds = 30
        self._interim_results = 'no'
        self._digit_norm = 'yes'
        self._vocabulary_id = None
        self._vad_threshold = 0
        self._user_words = []
        self._need_word_info = 'no'

    def set_add_punc(self, add_punc):
        self._add_punc = add_punc

    def set_digit_norm(self, digit_norm):
        self._digit_norm = digit_norm

    def set_vad_head(self, vad_head):
        self._vad_head = vad_head

    def set_vad_tail(self, vad_tail):
        self._vad_tail = vad_tail

    def set_max_seconds(self, max_seconds):
        self._max_seconds = max_seconds

    def set_interim_results(self, interim_results):
        self._interim_results = interim_results

    def set_vocabulary_id(self, vocabulary_id):
        self._vocabulary_id = vocabulary_id

    def set_vad_threshold(self, vad_threshold):
        self._vad_threshold = vad_threshold

    def set_user_words(self, user_words):
        self._user_words = user_words

    def set_need_word_info(self, need_word_info):
        self._need_word_info = need_word_info

    def construct_params(self):
        config = dict()
        config['audio_format'] = self._audio_format
        config['property'] = self._property
        config['add_punc'] = self._add_punc
        config['digit_norm'] = self._digit_norm
        config['vad_head'] = self._vad_head
        config['vad_tail'] = self._vad_tail
        config['max_seconds'] = self._max_seconds
        config['interim_results'] = self._interim_results
        config['vad_threshold'] = self._vad_threshold
        config['need_word_info'] = self._need_word_info
        if self._user_words is not None and len(self._user_words) > 0:
            config['user_words'] = self._user_words
        if self._vocabulary_id is not None:
            config['vocabulary_id'] = self._vocabulary_id

        params = dict()
        params['command'] = 'START'
        params['config'] = config
        return params



# -*- coding: utf-8 -*-

from hwsis.utils.logger_utils import logger
from hwsis.exception.exceptions import ClientException


class RasrCallBack:
    """ 实时语音识别的监听接口，监听创立链接、开始、中间响应、结束、关闭连接、错误 """
    def on_open(self):
        logger.debug('websocket connect success')

    def on_start(self, message):
        logger.debug('websocket start, %s' % message)

    def on_response(self, message):
        raise ClientException('no response implementation')

    def on_end(self, message):
        logger.debug('websocket end, %s' % message)

    def on_close(self):
        logger.debug('websocket close')

    def on_error(self, error):
        logger.error(error)


class RttsCallBack:
    def on_close(self):
        logger.debug('websocket close')

    def on_open(self):
        logger.debug('websocket connect success')

    def on_start(self, message):
        logger.debug('websocket start, %s' % message)

    def on_response(self, data):
        raise ClientException('no response implementation')

    def on_end(self, message):
        logger.debug('websocket end, %s' % message)

    def on_error(self, error):
        logger.error(error)



# -*- coding: utf-8 -*-


class ClientException(Exception):
    def __init__(self, error_msg):
        super().__init__(self, error_msg)
        self._error_msg = error_msg

    def get_error_msg(self):
        return self._error_msg

    def __str__(self):
        return self.get_error_msg()


class ServerException(Exception):
    def __init__(self, error_code, error_msg):
        dict1 = dict()
        dict1['error_code'] = error_code
        dict1['error_msg'] = error_msg
        super().__init__(self, str(dict1))
        self._error_code = error_code
        self._error_msg = error_msg

    def get_error_code(self):
        return self._error_code

    def get_error_msg(self):
        return self._error_msg

    def __str__(self):
        return 'error_code: ' + self.get_error_code() + '\t' + 'error_msg: ' + self.get_error_msg()







# -*- coding: utf-8 -*-


class TtsCustomRequest:
    """ 语音合成请求，除了初始化必选参数外，其他参数均可不配置使用默认 """
    def __init__(self, text):
        """
            语音合成请求初始化
        :param text: 需要合成的文本
        """
        self._text = text
        self._audio_format = 'wav'
        self._property = 'chinese_xiaoyan_common'
        self._sample_rate = '8000'
        self._speed = 0
        self._pitch = 0
        self._volume = 50
        self._saved = False
        self._saved_path = ''

    def set_audio_format(self, audio_format):
        self._audio_format = audio_format

    def set_volume(self, volume):
        self._volume = volume

    def set_sample_rate(self, sample_rate):
        self._sample_rate = sample_rate

    def set_property(self, model_property):
        self._property = model_property

    def set_pitch(self, pitch):
        self._pitch = pitch

    def set_speed(self, speed):
        self._speed = speed

    def set_saved(self, saved):
        self._saved = saved

    def set_saved_path(self, saved_path):
        self._saved_path = saved_path

    def get_saved(self):
        return self._saved

    def get_saved_path(self):
        return self._saved_path

    def construct_params(self):
        config = dict()
        config['audio_format'] = self._audio_format
        config['sample_rate'] = self._sample_rate
        config['property'] = self._property
        config['speed'] = self._speed
        config['pitch'] = self._pitch
        config['volume'] = self._volume

        params = dict()
        params['text'] = self._text
        params['config'] = config
        return params



# -*- coding: utf-8 -*-


class SisConfig:
    """ client配置参数，包括超时、代理 """
    def __init__(self):
        self.connect_timeout = 10
        self.read_timeout = 10
        self.connect_lost_timeout = 10
        self.proxy = None

    def set_connect_timeout(self, timeout):
        """
            设置连接超时
        :param timeout: seconds
        """
        self.connect_timeout = timeout

    def get_connect_timeout(self):
        """
            返回连接超时时间
        :return: connect_timeout, 单位秒
        """
        return self.connect_timeout

    def set_read_timeout(self, timeout):
        """
            设置读取超时
        :param timeout: seconds
        """
        self.read_timeout = timeout

    def get_read_timeout(self):
        """
            返回读取超时
        :return: read_timeout, 单位秒
        """
        return self.read_timeout

    def set_proxy(self, proxy):
        """
            设置代理
        :param proxy: 格式为list，[host, port] 或 [host, port, username, password]
        """
        self.proxy = proxy

    def get_proxy(self):
        """
            返回代理
        :return: proxy
        """
        return self.proxy

    def set_connect_lost_timeout(self, timeout):
        """
            设置connect lost 超时, 在并发满足要求下不需要设置此参数
        :param timeout: seconds
        """
        self.connect_lost_timeout = timeout

    def get_connect_lost_timeout(self):
        """
            返回connnect lost超时
        :return: connect_lost_timeout, 单位秒
        """
        return self.connect_lost_timeout



import logging.handlers

# 获取一个日志logger
logging.basicConfig(
    level=logging.INFO,
    format='[%(asctime)s] - [%(levelname)s] - [%(message)s]'
)
# 设置handler
file_handler = logging.handlers.RotatingFileHandler('hwsis.log', maxBytes=1024*1024,
                                                    backupCount=5, encoding='utf-8')
file_handler.setLevel(logging.INFO)
file_handler.setFormatter(logging.Formatter('[%(asctime)s] - [%(levelname)s] - [%(message)s]'))

# 添加file_handler
logger = logging.getLogger('hwsis')
logger.addHandler(file_handler)



# -*- coding: utf-8 -*-

import json
from hwsis.auth import signer
from hwsis.bean.sis_config import SisConfig
from hwsis.utils import http_utils
from hwsis.exception.exceptions import ClientException
from hwsis.utils.logger_utils import logger


def get_signed_headers(ak, sk, url, headers, params, http_method):
    """
        根据ak和sk以及请求信息获取加密头部
    @:return 加密的头部
    """
    sig = signer.Signer()
    sig.Key = ak
    sig.Secret = sk
    if params is None:
        body = ''
    else:
        body = params
        if isinstance(params, dict):
            body = json.dumps(params)
    r = signer.HttpRequest(http_method, url, headers, body)
    sig.Sign(r)
    return r.headers


def aksk_connect(ak, sk, url, headers, params, http_method, config=None):
    """
        根据url，返回json
    :param ak:  ak
    :param sk:  sk
    :param url: 完整请求url
    :param headers: 请求header，dict
    :param params:  请求参数， dict
    :param http_method: 请求方法，'POST' or 'GET', 其他会报错
    :param config: SisConfig(), 配置超时和代理
    :return: http返回结果转化为json
    """
    sis_config = config
    if sis_config is None:
        sis_config = SisConfig()
    if not isinstance(sis_config, SisConfig):
        error_msg = 'the param \'config\' in aksk_connect must be SisConfig class'
        logger.error(error_msg)
        raise ClientException(error_msg)
    signed_headers = get_signed_headers(ak, sk, url, headers, params, http_method)
    time_out = (sis_config.get_connect_timeout(), sis_config.get_read_timeout())
    resp = http_utils.http_connect(url, signed_headers, params, http_method, time_out, sis_config.get_proxy())
    json_result = http_utils.parse_resp(resp)
    if resp is not None:
        resp.close()
    return json_result
